﻿using System;
using System.Text;
using System.IO;
using System.Runtime.Serialization.Json;
using Com.Bocom.OpenApi.Security.Sign;
using System.Collections.Generic;

namespace Com.Bocom.OpenApi
{
    namespace Utils
    {
        public class ApiUtils
        {
            public static Object ParseNotifyJsonWithBocomSign(string notifyStr, string charset, string APIGW_PUBLIC_KEY,
                Type T)
            {
                string bizContentStr;
                string sign;
                try
                {
                    int indexOfRootStart = notifyStr.IndexOf(BocomConstants.NOTIFY_BIZ_CONTENT)
                                           + BocomConstants.NOTIFY_BIZ_CONTENT.Length + 2;
                    int indexOfRootEnd = notifyStr.LastIndexOf(",");
                    int indexOfSignStart =
                        notifyStr.LastIndexOf(BocomConstants.SIGN + "\"") + BocomConstants.SIGN.Length + 3;
                    int indexOfSignEnd = notifyStr.LastIndexOf("\"");
                    bizContentStr = notifyStr.Substring(indexOfRootStart, indexOfRootEnd - indexOfRootStart);
                    sign = notifyStr.Substring(indexOfSignStart, indexOfSignEnd - indexOfSignStart);
                }
                catch (Exception e)
                {
                    throw new Exception("notify_biz_content is not format json. notifyStr :\n" + notifyStr, e);
                }

                Boolean passed = RSASignature.Verify(bizContentStr, APIGW_PUBLIC_KEY, charset, sign);
                if (!passed)
                {
                    throw new Exception("Bocom sign verify not passed.");
                }

                try
                {
                    //反序列化并返回
                    BocomResponse response;
                    MemoryStream ms1 = new MemoryStream(Encoding.GetEncoding(charset).GetBytes(bizContentStr));
                    using (MemoryStream ms = new MemoryStream(Encoding.GetEncoding(charset).GetBytes(bizContentStr)))
                    {
                        DataContractJsonSerializer deseralizer = new DataContractJsonSerializer(T);
                        response = (BocomResponse) deseralizer.ReadObject(ms) as BocomResponse;
                    }

                    return response;
                }
                catch (Exception e)
                {
                    throw new Exception("notify_biz_content can not transform to defined class. response: " + notifyStr
                                                                                                            + " defined class name: " +
                                                                                                            T, e);
                }
            }

            public static Object ParseCommunicationJsonWithBocomSign(string respStr, string charset, string MY_PRIVATE_KEY, string APIGW_PUBLIC_KEY)
            {
                string bizContentStr;
                string sign;
                try
                {
                    //int indexOfRootStart = respStr.IndexOf(BocomConstants.BIZ_CONTENT) + BocomConstants.BIZ_CONTENT.Length + 2;
                    int indexOfRootEnd = respStr.LastIndexOf(",");
                    int indexOfSignStart = respStr.LastIndexOf(BocomConstants.SIGN + "\"") + BocomConstants.SIGN.Length + 3;
                    int indexOfSignEnd = respStr.LastIndexOf("\"");
                    bizContentStr = respStr.Substring(1, indexOfRootEnd - 1);
                    sign = respStr.Substring(indexOfSignStart, indexOfSignEnd - indexOfSignStart);
                }
                catch (Exception e)
                {
                    throw new Exception("response is not format json. respStr :\n" + respStr, e);
                }

                Boolean passed = BocomSignature.Verify(BocomConstants.ENCRYPT_TYPE_RSA_AND_AES, bizContentStr, APIGW_PUBLIC_KEY, charset, sign);
                if (!passed)
                {
                    throw new Exception("Bocom sign verify not passed.");
                }

                try
                {
                    var jsonDic = Kivii.Text.JsonSerializer.DeserializeFromString<Dictionary<string, string>>(respStr);
                    var encryptKeyEncrypt = jsonDic["encrypt_key"];
                    var reqBizContentEncrypt = jsonDic["biz_content"];
                    var encryptKey = BocomEncrypt.AsymmetricalDecrypt(encryptKeyEncrypt, BocomConstants.ENCRYPT_TYPE_RSA_AND_AES, MY_PRIVATE_KEY, charset);
                    var reqBizContent = BocomEncrypt.SymmetricalDecrypt(reqBizContentEncrypt, BocomConstants.ENCRYPT_TYPE_RSA_AND_AES, encryptKey, charset);
                    jsonDic["encrypt_key"] = encryptKey;
                    jsonDic["biz_content"] = reqBizContent;
                    //var rtns=Kivii.Text.JsonSerializer.SerializeToString(jsonDic);
                    return jsonDic;
                }
                catch (Exception e)
                {
                    throw new Exception("Bocom decrypt not success.", e);
                }
            }
        }
    }
}